// Edited 3/17/2015

/**************************************************************************/
/*!
    @file     Adafruit_ADS1015.cpp
    @author   K.Townsend (Adafruit Industries)
    @license  BSD (see LICENSE.txt)
 
    Ported to mbed by Arve Seljebu - arve0.github.io
 
    Driver for the ADS1015/ADS1115 ADC
 
    This is a library for the Adafruit MPL115A2 breakout
    ----> https://www.adafruit.com/products/1083
 
    Adafruit invests time and resources providing this open source code,
    please support Adafruit and open-source hardware by purchasing
    products from Adafruit!
 
    @section  HISTORY
 
    v1.0 - First release
    v1.1.1 - Ported to mbed
*/
/**************************************************************************/
#include "simpletools.h"
#include "simplei2c.h"
#include "config.h"
#include "adafruit_ads1015.h"

// The number of value pairs in the table.
#define TABLE_SIZE 53
//
// This is a table of ADC readings and corresponding temperature values for both the
// extruder and heated build platform.  In determining the thermistor resistance values,
// data was extracted from the EPCOS B57560G1104F000 datasheet, because that is the
// thermistor being utilized for both the extruder and heated build platform.
//
// During the calculations of this table, a resistor having a nominal value of 4.7K,
// with a 10% tolerance was utilized, however the calculations were performed and based
// upon the actual resistance, which was 4,602 Ohms.
LOOKUPTEMP lookup_temp[TABLE_SIZE] = {
	{.sample = 1591, .temp = 20},
	{.sample = 1577, .temp = 25},
	{.sample = 1560, .temp = 30},
	{.sample = 1540, .temp = 35},
	{.sample = 1517, .temp = 40},
	{.sample = 1490, .temp = 45},
	{.sample = 1459, .temp = 50},
	{.sample = 1424, .temp = 55},
	{.sample = 1386, .temp = 60},
	{.sample = 1343, .temp = 65},
	{.sample = 1296, .temp = 70},
	{.sample = 1245, .temp = 75},
	{.sample = 1191, .temp = 80},
	{.sample = 1135, .temp = 85},
	{.sample = 1076, .temp = 90},
	{.sample = 1016, .temp = 95},
	{.sample = 956, .temp = 100},
	{.sample = 895, .temp = 105},
	{.sample = 835, .temp = 110},
	{.sample = 776, .temp = 115},
	{.sample = 719, .temp = 120},
	{.sample = 665, .temp = 125},
	{.sample = 612, .temp = 130},
	{.sample = 563, .temp = 135},
	{.sample = 517, .temp = 140},
	{.sample = 474, .temp = 145},
	{.sample = 434, .temp = 150},
	{.sample = 397, .temp = 155},
	{.sample = 362, .temp = 160},
	{.sample = 331, .temp = 165},
	{.sample = 302, .temp = 170},
	{.sample = 276, .temp = 175},
	{.sample = 252, .temp = 180},
	{.sample = 231, .temp = 185},
	{.sample = 211, .temp = 190},
	{.sample = 193, .temp = 195},
	{.sample = 177, .temp = 200},
	{.sample = 162, .temp = 205},
	{.sample = 149, .temp = 210},
	{.sample = 136, .temp = 215},
	{.sample = 125, .temp = 220},
	{.sample = 115, .temp = 225},
	{.sample = 106, .temp = 230},
	{.sample = 98, .temp = 235},
	{.sample = 90, .temp = 240},
	{.sample = 83, .temp = 245},
	{.sample = 77, .temp = 250},
	{.sample = 71, .temp = 255},
	{.sample = 66, .temp = 260},
	{.sample = 61, .temp = 265},
	{.sample = 57, .temp = 270},
	{.sample = 53, .temp = 275},
	{.sample = 49, .temp = 280},
};

double extruder_celsius;
double bed_celsius;

/**************************************************************************/
/*!
    @brief  Instantiates a new ADS1015 class w/appropriate properties
*/
/**************************************************************************/
void adc_init()
{
    uint8_t i2cAddress;
    
    m_i2c = i2c_newbus(I2C_SCL,  I2C_SDA, I2C_MODE);
    i2cAddress = ADS1015_ADDRESS;    
    m_i2cAddress = i2cAddress;
    m_conversionDelay = ADS1015_CONVERSIONDELAY;
    m_bitShift = 4;
    m_gain = GAIN_ONE; // +/-4.096V range = Gain 1
}

/**************************************************************************/
/*!
    @brief  Given the appropriate channel, update the proper external
    temperature variables.
*/
/**************************************************************************/
void UpdateTemp(uint8_t channel)
{
     double samples[NUMSAMPLES];
	uint8_t i;
	double average; 

	// take N samples in a row, with a slight delay
	for (i = 0; i < NUMSAMPLES; i++)
	{
          // read channel the specified channel
		samples[i] = readADC_SingleEnded(channel);
          pause(10);
	}

	// average all the samples out
	average = 0;

	for(i = 0; i < NUMSAMPLES; i++)
	{
		average += samples[i];
	}

	average /= NUMSAMPLES;   

	if(channel == 0)
	{
     	extruder_celsius = lookupTemp(lookup_temp, average);
	}
	if(channel == 1)
	{
     	bed_celsius = lookupTemp(lookup_temp, average);
	}
}

/**************************************************************************/
/*!
    @brief  Given an ADC sample average, lookup the appropriate temperature.
*/
/**************************************************************************/
double lookupTemp(const LOOKUPTEMP *table, double sample)
{
	int index;  

	index = 0;

	// find the two points in the table to use
	while(index < TABLE_SIZE && sample < table[index].sample)
     {
		index++;
	}

	// make sure the point isn't past the end of the table
	if( index == TABLE_SIZE)
	{
		return table[index - 1].temp;
	}

	// make sure the point isn't before the beginning of the table 
	if(index == 0)
	{
		return table[index].temp;
	}

	return table[index].temp;
}
 
/**************************************************************************/
/*!
    @brief  Writes 16-bits to the specified destination register
*/
/**************************************************************************/
void writeRegister(uint8_t i2cAddress, uint8_t reg, uint16_t value)
{
    unsigned char cmd[3];
    cmd[0] = (char)reg;
    cmd[1] = (char)(value>>8);
    cmd[2] = (char)(value & 0xFF);
       
    i2c_out(m_i2c, i2cAddress, 0, 0, cmd, 3);
} 

/**************************************************************************/
/*!
    @brief  Reads 16-bits from the specified register
*/
/**************************************************************************/
uint16_t readRegister(uint8_t i2cAddress, uint8_t reg)
{
    unsigned char data[2];
    data[0] = reg; // temporary use this to send address to conversion register
    
    i2c_out(m_i2c, i2cAddress, 0, 0, data, 1);
    i2c_in(m_i2c, i2cAddress, 0, 0, data, 2);

    return (data[0] << 8 | data [1]);
}
 
/**************************************************************************/
/*!
    @brief  Sets the gain and input voltage range
*/
/**************************************************************************/
void setGain(adsGain_t gain)
{
    m_gain = gain;
}
 
/**************************************************************************/
/*!
    @brief  Gets a gain and input voltage range
*/
/**************************************************************************/
adsGain_t getGain()
{
    return m_gain;
}
 
/**************************************************************************/
/*!
    @brief  Gets a single-ended ADC reading from the specified channel
*/
/**************************************************************************/
uint16_t readADC_SingleEnded(uint8_t channel)
{
    if (channel > 3) {
        return 0;
    }
 
    // Start with default values
    uint16_t config = ADS1015_REG_CONFIG_CQUE_NONE    | // Disable the comparator (default val)
                      ADS1015_REG_CONFIG_CLAT_NONLAT  | // Non-latching (default val)
                      ADS1015_REG_CONFIG_CPOL_ACTVLOW | // Alert/Rdy active low   (default val)
                      ADS1015_REG_CONFIG_CMODE_TRAD   | // Traditional comparator (default val)
                      ADS1015_REG_CONFIG_DR_1600SPS   | // 1600(ADS1015) or 250(ADS1115) samples per second (default)
                      ADS1015_REG_CONFIG_MODE_SINGLE;   // Single-shot mode (default)
 
    // Set PGA/voltage range
    config |= m_gain;
 
    // Set single-ended input channel
    switch (channel) {
        case (0):
            config |= ADS1015_REG_CONFIG_MUX_SINGLE_0;
            break;
        case (1):
            config |= ADS1015_REG_CONFIG_MUX_SINGLE_1;
            break;
        case (2):
            config |= ADS1015_REG_CONFIG_MUX_SINGLE_2;
            break;
        case (3):
            config |= ADS1015_REG_CONFIG_MUX_SINGLE_3;
            break;
    }
 
    // Set 'start single-conversion' bit
    config |= ADS1015_REG_CONFIG_OS_SINGLE;
 
    // Write config register to the ADC
    writeRegister(m_i2cAddress, ADS1015_REG_POINTER_CONFIG, config);
 
    // Wait for the conversion to complete
    pause(m_conversionDelay);
 
    // Read the conversion results
    // Shift 12-bit results right 4 bits for the ADS1015
    return readRegister(m_i2cAddress, ADS1015_REG_POINTER_CONVERT) >> m_bitShift;
}
 
/**************************************************************************/
/*!
    @brief  Reads the conversion results, measuring the voltage
            difference between the P (AIN0) and N (AIN1) input.  Generates
            a signed value since the difference can be either
            positive or negative.
*/
/**************************************************************************/
int16_t readADC_Differential_0_1()
{
    // Start with default values
    uint16_t config = ADS1015_REG_CONFIG_CQUE_NONE    | // Disable the comparator (default val)
                      ADS1015_REG_CONFIG_CLAT_NONLAT  | // Non-latching (default val)
                      ADS1015_REG_CONFIG_CPOL_ACTVLOW | // Alert/Rdy active low   (default val)
                      ADS1015_REG_CONFIG_CMODE_TRAD   | // Traditional comparator (default val)
                      ADS1015_REG_CONFIG_DR_1600SPS   | // 1600(ADS1015) or 250(ADS1115) samples per second (default)
                      ADS1015_REG_CONFIG_MODE_SINGLE;   // Single-shot mode (default)
 
    // Set PGA/voltage range
    config |= m_gain;
 
    // Set channels
    config |= ADS1015_REG_CONFIG_MUX_DIFF_0_1;          // AIN0 = P, AIN1 = N
 
    // Set 'start single-conversion' bit
    config |= ADS1015_REG_CONFIG_OS_SINGLE;
 
    // Write config register to the ADC
    writeRegister(m_i2cAddress, ADS1015_REG_POINTER_CONFIG, config);
 
    // Wait for the conversion to complete
    pause(m_conversionDelay);
 
    // Read the conversion results
    uint16_t res = readRegister(m_i2cAddress, ADS1015_REG_POINTER_CONVERT) >> m_bitShift;
    if (m_bitShift == 0) {
        return (int16_t)res;
    } else {
        // Shift 12-bit results right 4 bits for the ADS1015,
        // making sure we keep the sign bit intact
        if (res > 0x07FF) {
            // negative number - extend the sign to 16th bit
            res |= 0xF000;
        }
        return (int16_t)res;
    }
}
 
/**************************************************************************/
/*!
    @brief  Reads the conversion results, measuring the voltage
            difference between the P (AIN2) and N (AIN3) input.  Generates
            a signed value since the difference can be either
            positive or negative.
*/
/**************************************************************************/
int16_t readADC_Differential_2_3()
{
    // Start with default values
    uint16_t config = ADS1015_REG_CONFIG_CQUE_NONE    | // Disable the comparator (default val)
                      ADS1015_REG_CONFIG_CLAT_NONLAT  | // Non-latching (default val)
                      ADS1015_REG_CONFIG_CPOL_ACTVLOW | // Alert/Rdy active low   (default val)
                      ADS1015_REG_CONFIG_CMODE_TRAD   | // Traditional comparator (default val)
                      ADS1015_REG_CONFIG_DR_1600SPS   | // 1600(ADS1015) or 250(ADS1115) samples per second (default)
                      ADS1015_REG_CONFIG_MODE_SINGLE;   // Single-shot mode (default)
 
    // Set PGA/voltage range
    config |= m_gain;
 
    // Set channels
    config |= ADS1015_REG_CONFIG_MUX_DIFF_2_3;          // AIN2 = P, AIN3 = N
 
    // Set 'start single-conversion' bit
    config |= ADS1015_REG_CONFIG_OS_SINGLE;
 
    // Write config register to the ADC
    writeRegister(m_i2cAddress, ADS1015_REG_POINTER_CONFIG, config);
 
    // Wait for the conversion to complete
    pause(m_conversionDelay);
 
    // Read the conversion results
    uint16_t res = readRegister(m_i2cAddress, ADS1015_REG_POINTER_CONVERT) >> m_bitShift;
    if (m_bitShift == 0) {
        return (int16_t)res;
    } else {
        // Shift 12-bit results right 4 bits for the ADS1015,
        // making sure we keep the sign bit intact
        if (res > 0x07FF) {
            // negative number - extend the sign to 16th bit
            res |= 0xF000;
        }
        return (int16_t)res;
    }
}
 
/**************************************************************************/
/*!
    @brief  Sets up the comparator to operate in basic mode, causing the
            ALERT/RDY pin to assert (go from high to low) when the ADC
            value exceeds the specified threshold.
 
            This will also set the ADC in continuous conversion mode.
*/
/**************************************************************************/
void startComparator_SingleEnded(uint8_t channel, int16_t threshold)
{
    // Start with default values
    uint16_t config = ADS1015_REG_CONFIG_CQUE_1CONV   | // Comparator enabled and asserts on 1 match
                      ADS1015_REG_CONFIG_CLAT_LATCH   | // Latching mode
                      ADS1015_REG_CONFIG_CPOL_ACTVLOW | // Alert/Rdy active low   (default val)
                      ADS1015_REG_CONFIG_CMODE_TRAD   | // Traditional comparator (default val)
                      ADS1015_REG_CONFIG_DR_1600SPS   | // 1600(ADS1015) or 250(ADS1115) samples per second (default)
                      ADS1015_REG_CONFIG_MODE_CONTIN  | // Continuous conversion mode
                      ADS1015_REG_CONFIG_MODE_CONTIN;   // Continuous conversion mode
 
    // Set PGA/voltage range
    config |= m_gain;
 
    // Set single-ended input channel
    switch (channel) {
        case (0):
            config |= ADS1015_REG_CONFIG_MUX_SINGLE_0;
            break;
        case (1):
            config |= ADS1015_REG_CONFIG_MUX_SINGLE_1;
            break;
        case (2):
            config |= ADS1015_REG_CONFIG_MUX_SINGLE_2;
            break;
        case (3):
            config |= ADS1015_REG_CONFIG_MUX_SINGLE_3;
            break;
    }
 
    // Set the high threshold register
    // Shift 12-bit results left 4 bits for the ADS1015
    writeRegister(m_i2cAddress, ADS1015_REG_POINTER_HITHRESH, threshold << m_bitShift);
 
    // Write config register to the ADC
    writeRegister(m_i2cAddress, ADS1015_REG_POINTER_CONFIG, config);
}
 
/**************************************************************************/
/*!
    @brief  In order to clear the comparator, we need to read the
            conversion results.  This function reads the last conversion
            results without changing the config value.
*/
/**************************************************************************/
int16_t getLastConversionResults()
{
    // Wait for the conversion to complete
    pause(m_conversionDelay);
 
    // Read the conversion results
    uint16_t res = readRegister(m_i2cAddress, ADS1015_REG_POINTER_CONVERT) >> m_bitShift;
    if (m_bitShift == 0) {
        return (int16_t)res;
    } else {
        // Shift 12-bit results right 4 bits for the ADS1015,
        // making sure we keep the sign bit intact
        if (res > 0x07FF) {
            // negative number - extend the sign to 16th bit
            res |= 0xF000;
        }
        return (int16_t)res;
    }
}